﻿using C1.WPF;
using C1.WPF.DataGrid;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace DataGridDragDrop
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private ViewModel _viewModel;
        private C1DragDropManager _dragDropManager;
        private ObservableCollection<User> draggedData;

        public MainWindow()
        {
            InitializeComponent();
            _viewModel = new ViewModel();
            DataContext = _viewModel;

            draggedData = new ObservableCollection<User>();
            _dragDropManager = new C1DragDropManager();
            _dragDropManager.DragDrop += OnDragDrop;
            _dragDropManager.DragOver += _dragDropManager_DragOver;
            _dragDropManager.RegisterDropTarget(targetDataGrid, true);
            _dragDropManager.Canvas.Opacity = 0.4;
            sourceDataGrid.PreviewMouseDown += Source_PreviewMouseDown;
        }

        private void Source_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
                var cell = sourceDataGrid.GetCellFromPoint(e.GetPosition(null));
                if (cell != null && cell.IsSelected)
                {
                    draggedData.Clear();
                    foreach (User item in sourceDataGrid.SelectedItems)
                    {
                        draggedData.Add(item);
                    }
                    _dragDropManager.DoDragDrop(cell.Row.Presenter, e, DragDropEffect.Move);
                e.Handled = true;
            }                       
        }

        private void _dragDropManager_DragOver(object source, DragDropEventArgs e)
        {
            var targetDataGrid = e.DropTarget as C1DataGrid;
            var cell = targetDataGrid.GetCellFromPoint(Mouse.GetPosition(this));
            if(cell != null)
            {
                targetDataGrid.Selection.Clear();
                targetDataGrid.Selection.Add(new DataGridCellsRange(cell.Row));
            }
            ((_dragDropManager.Canvas.Children[2] as Border).Child as Border).Height =sourceDataGrid.RowHeight.Value*draggedData.Count;
        }

        private void OnDragDrop(object source, DragDropEventArgs e)
        {

            var targetDataGrid = e.DropTarget as C1DataGrid;
            var cell = targetDataGrid.GetCellFromPoint(Mouse.GetPosition(this));
            if(cell!=null)
            {
                int index = cell.Row.Index;
                foreach (User item in draggedData)
                {
                    (sourceDataGrid.DataSourceView.SourceCollection as ObservableCollection<User>).Remove(item);
                    (targetDataGrid.DataSourceView.SourceCollection as ObservableCollection<User>).Insert(index, item);
                    index++;
                }
                targetDataGrid.Selection.BeginUpdate();
                targetDataGrid.Selection.Clear();
                targetDataGrid.Selection.Add(targetDataGrid.Rows[cell.Row.Index - draggedData.Count], targetDataGrid.Rows[cell.Row.Index - 1]);
                targetDataGrid.Selection.EndUpdate();
                draggedData.Clear();
            }
        }

    }
}
